'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Mixin = {};
Mixin.CFG = {
  labelEditable: false
};
Mixin.INIT = '_initLabelEditor';
Mixin.AUGMENT = {
  _initLabelEditor: function _initLabelEditor() {
    var _this = this;

    var labelEditable = this.get('labelEditable');
    if (labelEditable) {
      var graph = this.getGraph();
      var labelTextArea = _util2.default.createDOM('<div contenteditable="true" role="textbox" tabindex="1" class="g6-label-editor"></div>', {
        position: 'absolute',
        visibility: 'hidden',
        'z-index': '2',
        padding: '0px 2px 0px 0px',
        resize: 'none',
        width: 'auto',
        height: 'auto',
        outline: 'none',
        top: '0',
        left: '0',
        border: '1px solid #1890FF',
        'transform-origin': 'left top',
        'max-width': '320px',
        background: 'white',
        'box-sizing': 'content-box'
      });
      graph.getGraphContainer().appendChild(labelTextArea);
      this.isCompositing = false;
      labelTextArea.on('compositionstart', function () {
        _this.isCompositing = true;
      });
      labelTextArea.on('compositionend', function () {
        _this.isCompositing = false;
      });
      labelTextArea.on('blur', function (ev) {
        ev.stopPropagation();
        if (!graph.destroyed) {
          _this.endEditLabel();
        }
      });
      labelTextArea.on('keydown', function (ev) {
        ev.stopPropagation();
        var key = _util2.default.getKeyboradKey(ev);
        if (ev.metaKey && key === 's' || ev.ctrlKey && key === 's') {
          ev.preventDefault();
        }
        if ((key === 'Enter' || key === 'Escape') && !_this.isCompositing) {
          _this.endEditLabel();
        }
      });
      this.set('labelTextArea', labelTextArea);
      graph.on('beforeviewportchange', function () {
        if (labelTextArea.focusItem) {
          _this.setLabelEditorBeginPosition(labelTextArea.focusItem);
        }
      });
    }
  },
  _getLabelTextAreaBox: function _getLabelTextAreaBox(labelShape, labelText) {
    var padding = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [0, 0];

    if (labelText) {
      labelShape.attr('text', labelText);
    }

    var graph = this.getGraph();
    var rootGroup = graph.getRootGroup();
    var labelBox = _util2.default.getBBox(labelShape, rootGroup);
    return {
      minX: labelBox.minX - padding[1],
      minY: labelBox.minY - padding[0],
      maxX: labelBox.maxX + padding[1],
      maxY: labelBox.maxY + padding[0]
    };
  },
  setLabelEditorBeginPosition: function setLabelEditorBeginPosition(item) {
    var labelTextArea = this.get('labelTextArea');
    var labelShape = item.getLabel();

    if (labelShape) {
      var labelBox = this._getLabelTextAreaBox(labelShape);

      var lineHeight = labelShape.attr('lineHeight');
      var fontSize = labelShape.attr('fontSize');
      var labelPoint = {
        x: labelBox.minX,
        y: labelBox.minY - lineHeight / 4 + fontSize / 4 - 1,
        width: labelBox.maxX - labelBox.minX,
        height: labelBox.maxY - labelBox.minY
      };
      labelTextArea.css({
        top: labelPoint.y + 'px',
        left: labelPoint.x + 'px'
      });
      labelTextArea.labelPoint = labelPoint;
    } else {
      var graph = this.getGraph();
      var rootGroup = graph.getRootGroup();
      var keyShape = item.getKeyShape();
      var keyShapeBox = _util2.default.getBBox(keyShape, rootGroup);
      var _labelPoint = {
        x: keyShapeBox.minY + (keyShapeBox.maxY - keyShapeBox.minY - labelTextArea.height()) / 2,
        y: (keyShapeBox.minX + keyShapeBox.maxX) / 2
      };
      labelTextArea.css({
        top: _labelPoint.x + 'px',
        left: _labelPoint.y + 'px'
      });
      labelTextArea.labelPoint = _labelPoint;
    }
  },
  beginEditLabel: function beginEditLabel(item) {
    var labelTextArea = this.get('labelTextArea');
    var graph = this.getGraph();

    if (_util2.default.isString(item)) {
      item = graph.find(item);
    }

    if (item && !item.destroyed && labelTextArea) {
      this.setSignal('preventWheelPan', true);
      var model = item.getModel();
      var labelShape = item.getLabel();
      var zoom = graph.getZoom();
      labelTextArea.focusItem = item;

      if (labelShape) {
        var lineHeight = labelShape.attr('lineHeight');
        var labelBox = this._getLabelTextAreaBox(labelShape);
        var minWidth = (labelBox.maxX - labelBox.minX) / zoom;
        var minHeight = (labelBox.maxY - labelBox.minY + lineHeight / 4) / zoom;
        labelTextArea.innerHTML = model.label;
        labelTextArea.css({
          'min-width': minWidth + 'px',
          'min-height': minHeight + 'px',
          visibility: 'visible',
          'font-family': labelShape.attr('fontFamily'),
          'line-height': lineHeight + 'px',
          'font-size': labelShape.attr('fontSize') + 'px',
          transform: 'scale(' + zoom + ')'
        });
      } else {
        labelTextArea.innerHTML = '';
        labelTextArea.css({
          'min-width': 'auto',
          'min-height': 'auto'
        });
      }

      this.setLabelEditorBeginPosition(item);
      labelTextArea.css({
        visibility: 'visible'
      });
      labelTextArea.focus();
      document.execCommand('selectAll', false, null);
    }
  },
  endEditLabel: function endEditLabel() {
    var labelTextArea = this.get('labelTextArea');
    this.setSignal('preventWheelPan', false);

    if (labelTextArea) {
      var item = labelTextArea.focusItem;

      if (item && !item.destroyed) {
        var model = item.getModel();
        var editor = this.editor;

        if (model.label !== labelTextArea.textContent) {
          editor.executeCommand('update', {
            action: 'updateLabel',
            itemId: item.id,
            updateModel: {
              label: labelTextArea.textContent
            }
          });
        }

        labelTextArea.hide();
        labelTextArea.focusItem = undefined;
        this.focusGraphWrapper();
      }
    }
  },
  cancelEditLabel: function cancelEditLabel() {
    var labelTextArea = this.get('labelTextArea');
    labelTextArea.focusItem = undefined;
    labelTextArea.hide();
  }
};
exports.default = Mixin;