'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getContentArea = exports.fixDragEvent = exports.normalizeTdContent = exports.unWrapperCodeBlock = exports.unWrapperTableCommon = exports.unWrapperTableBI = exports.unWrapperTable = exports.unWrapperTableHTML = exports.getCopyData = exports.copyHTML = exports.copyCss = exports.copyTo = exports.trimBlankSpan = exports.tableInnerSchema = exports.trimStartTr = exports.normalizeTable = exports.getTableModel = undefined;

var _isInteger = require('lodash/isInteger');

var _isInteger2 = _interopRequireDefault(_isInteger);

var _isObject = require('lodash/isObject');

var _isObject2 = _interopRequireDefault(_isObject);

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var $ = _tinsoEditorEngine2.default.$,
    StringUtils = _tinsoEditorEngine2.default.StringUtils,
    NodeUtils = _tinsoEditorEngine2.default.NodeUtils;

/**
 * 提取表格数据模型，由于合并单元格的存在，html 结构不利于操作
 * @param {NativeNode} table 表格原生对象
 * @return {(tdModel[])[]} result 一个二维数组
 * [
 *  [tdModel, tdModel...],
 *  [tdModel, tdModel...],
 *  ...
 * ]
 * @tdModel:
 *  {
 *    isMulti: {boolean} 合并的单元格
 *    isEmpty: {boolean} 为 true 时表示被合并单元格覆盖到的占位
 *    isShadow: {boolean} 为 true 时表示这是一个补充的单元格，html拷贝的时候会出现遗漏单元格，这里需要补充上
 *    parent: {row, col} 标记占位格的父单元格的坐标位置
 *    element: {NativeNode} 指针作用，指向对应的 td
 *    rowSpan: {number} 单元格的 rowSpan
 *    colSpan: {number} 单元格的 colSpan
 *  }
 */

var getTableModel = exports.getTableModel = function getTableModel(table) {
  var model = [];
  var rows = table.rows;
  var rowCount = rows.length;

  for (var r = 0; r < rowCount; r++) {
    var tr = rows[r];
    var cells = tr.cells;
    var cellCount = cells.length;

    for (var c = 0; c < cellCount; c++) {
      var td = cells[c];
      var rowSpan = td.rowSpan,
          colSpan = td.colSpan;

      rowSpan = rowSpan === void 0 ? 1 : rowSpan;
      colSpan = colSpan === void 0 ? 1 : colSpan;
      var isMulti = rowSpan > 1 || colSpan > 1;
      model[r] = model[r] || [];
      // 如果当前单元格的 model 已经存在，说明是前面已经有合并单元格覆盖了当前坐标，需要往右推移
      var _c = c;
      while (model[r][_c]) {
        _c++;
      }

      model[r][_c] = {
        rowSpan: rowSpan,
        colSpan: colSpan,
        isMulti: isMulti,
        element: td
      };

      if (isMulti) {
        // 补齐被合并的单元格占位
        var _rowCount = rowSpan;

        while (_rowCount > 0) {
          var colCount = colSpan;
          while (colCount > 0) {
            if (colCount !== 1 || _rowCount !== 1) {
              var rowIndex = r + _rowCount - 1;
              var colIndex = _c + colCount - 1;
              model[rowIndex] = model[rowIndex] || [];
              model[rowIndex][colIndex] = {
                isEmpty: true,
                parent: {
                  row: r,
                  col: _c
                },
                element: null
              };
            }
            colCount--;
          }
          _rowCount--;
        }
      }
    }
  }

  var colCounts = model.map(function (trModel) {
    return trModel.length;
  });
  var MaxColCount = Math.max.apply(Math, [].concat(_toConsumableArray(colCounts)));
  model.forEach(function (trModel) {
    if (trModel.length < MaxColCount) {
      var addCount = MaxColCount - trModel.length;
      while (addCount--) {
        trModel.push({
          rowSpan: 1,
          colSpan: 1,
          isShadow: true,
          element: null
        });
      }
    }
    // 表格内有空数组项，补齐为 shadow
    for (var i = 0; i < MaxColCount; i++) {
      if (!trModel[i]) {
        trModel[i] = {
          rowSpan: 1,
          colSpan: 1,
          isShadow: true,
          element: null
        };
      }
    }
  });
  var result = {
    rows: model.length,
    cols: MaxColCount,
    width: table.offsetWidth,
    table: model
  };
  return result;
};
/**
 * table 结构标准化，补齐丢掉的单元格和行
 * 场景1. number 拷贝过来的 html 中，如果这一行没有单元格，就会省掉 tr，渲染的时候会有问题
 * 场景2. 从网页中鼠标随意选取表格中的一部分，会丢掉没有选中的单元格，需要补齐单元格
 * @param {nativeNode} table 表格 Dom
 * @return {nativeNode} 修复过的 table dom
 */

var normalizeTable = exports.normalizeTable = function normalizeTable(table) {
  var $table = $(table);
  trimStartTr($table);
  fixNumberTr(table);
  normalizeTdContent($table);
  $table.addClass('zacheditor-table');
  // 修正表格宽度为 pt 场景
  var width = $table.css('width');

  if (parseInt(width) === 0) {
    $table.css('width', 'auto');
  } else {
    // pt 直接转为 px, 因为 col 的 width 属性是没有单位的，会直接被理解为 px, 这里 table 的 width 也直接换成 px。
    $table.css('width', parseInt(width, 10) + 'px');
  } // 表格 table 标签不允许有背景色，无法设置

  $table.css('background-color', '');
  var model = getTableModel(table);
  // 修正列的 span 场景
  var cols = $table.find('col');
  if (cols.length !== 0) {
    for (var c = cols.length - 1; c >= 0; c--) {
      var _width = $(cols[c]).attr('width');
      if (_width) {
        $(cols[c]).attr('width', parseInt(_width));
      }
      if (cols[c].span > 1) {
        var addCount = cols[c].span - 1;
        while (addCount--) {
          cols[c].parentNode.insertBefore(cols[c].cloneNode(), cols[c]);
        }
      }
    }
    cols = $table.find('col');
    if (cols.length < model.cols) {
      var lastCol = cols.length - 1;
      var colsAddCount = model.cols - cols.length;
      while (colsAddCount--) {
        cols[0].parentNode.appendChild(cols[lastCol].cloneNode());
      }
    }
    $table.find('col').attr('span', 1);
  } else {
    var colgroup = $table.find('colgroup')[0];
    if (!colgroup) {
      colgroup = document.createElement('colgroup');
    }
    $table.prepend(colgroup);
    var widths = function (table) {
      var tr = table.find('tr')[0];
      var tds = $(tr).find('td');
      var widthArray = [];
      tds.each(function (td) {
        var colWidth = $(td).attr('data-colwidth');
        var tdWidth = $(td).attr('width');
        var tdColSpan = $(td)[0].colSpan;
        if (colWidth) {
          colWidth = colWidth.split(',');
        } else if (tdWidth) {
          tdWidth = parseInt(tdWidth) / tdColSpan;
        }
        for (var o = 0; tdColSpan > o; o++) {
          if (colWidth && colWidth[o]) {
            widthArray.push(parseInt(colWidth[o]));
          } else if (tdWidth) {
            widthArray.push(parseInt(tdWidth));
          } else {
            widthArray.push(undefined);
          }
        }
      });
      var td = table.find('td');
      td.removeAttr('data-colwidth');
      td.removeAttr('width');
      return widthArray;
    }($table);
    var col = document.createElement('col');
    for (var f = 0; model.cols > f; f++) {
      var node = col.cloneNode();
      if (widths[f]) {
        node.setAttribute('width', widths[f]);
      }
      colgroup.appendChild(node);
    }
  }
  // 数据模型和实际 dom 结构的行数不一致，需要寻找并补齐行
  model.table.forEach(function (tr, r) {
    if (!table.rows[r]) {
      table.insertRow(r);
    }
    var shadow = tr.filter(function (td) {
      return td.isShadow;
    });
    var shadowCount = shadow.length;
    while (shadowCount--) {
      if (r === 0) {
        table.rows[r].insertCell(0);
      } else {
        table.rows[r].insertCell();
      }
    }
  });
  // 修正行高
  var trs = $table.find('tr');
  trs.each(function (tr) {
    var $tr = $(tr);
    var height = $(tr).css('height');
    height = parseInt(height) || 33;
    $tr.css('height', height + 'px');
  });
  return table;
};

/**
 * 过滤 table 中的首行空tr, 当表格尾部有空白单元格时，网页拷贝时头部会莫名其妙的出现一个空的Tr
 * @param {nodeModel} table 传入的table
 */
var trimStartTr = exports.trimStartTr = function trimStartTr(table) {
  var tr = table.find('tr');
  if (tr[0].childNodes.length === 0) {
    $(tr[0]).remove();
  }
};

/**
 * 过滤 schema 中的 table 相关标签
 * @param {Array} schema 传入的schema
 * @return {Array} 过滤之后的schema
 */
var tableInnerSchema = exports.tableInnerSchema = function tableInnerSchema(schema) {
  return schema.filter(function (rule) {
    var allowed = true;
    if ((0, _isObject2.default)(rule)) {
      if (rule.table || rule.tr || rule.td || rule.th || rule.tbody || rule.col || rule.thead || rule.h1 || rule.h2 || rule.h3 || rule.h4 || rule.h5 || rule.h6) {
        allowed = false;
      }
    } else {
      if (rule === 'table' || rule === 'tr' || rule === 'td' || rule === 'th' || rule === 'col' || rule === 'colgroup' || rule === 'tbody' || rule === 'thead') {
        allowed = false;
      }
    }
    return allowed;
  });
};

/**
 * 去掉头尾空span
 * @param {nodeModel} node 传入的 node
 * @return {nodeModel} 去过头尾的 node
 */
var trimBlankSpan = exports.trimBlankSpan = function trimBlankSpan(node) {
  var len = node.length;
  var nodelist = [];
  var i = 0;
  var j = len - 1;

  while (node[i] && node[i].tagName.toLowerCase() === 'span' && node[i].innerText.trim() === '') {
    i++;
  }

  while (node[j] && node[j].tagName.toLowerCase() === 'span' && node[j].innerText.trim() === '') {
    j--;
  }

  if (i <= j) {
    for (var k = i; k <= j; k++) {
      nodelist.push(node[k]);
    }
  }

  if (nodelist.length) {
    return $(nodelist);
  }
  return node;
};

var copyTo = exports.copyTo = function copyTo(from, to) {
  var $to = $(to);
  var $from = $(from);
  $to.html(StringUtils.transformCustomTags(from.innerHTML));
  copyCss($from, $to);
};

var copyCss = exports.copyCss = function copyCss(from, to) {
  var $to = $(to);
  var $from = $(from);
  $to.css('text-align', $from.css('text-align'));
  $to.css('vertical-align', $from.css('vertical-align'));
  var tdBgColor = $from.css('background-color');
  tdBgColor = tdBgColor !== 'rgba(0, 0, 0, 0)' ? tdBgColor : '#fff';
  $to.css('background-color', tdBgColor);
  $to.css('color', $from.css('color'));
  $to.css('font-weight', $from.css('font-weight'));
};

var clipboard = null;
var copyHTML = exports.copyHTML = function copyHTML(html) {
  clipboard = {
    html: html,
    text: $(html).innerText
  };
};

var getCopyData = exports.getCopyData = function getCopyData() {
  return clipboard;
};

var unWrapperTableHTML = exports.unWrapperTableHTML = function unWrapperTableHTML(html) {
  html = html.replace(/<p(>|\s+[^>]*>)/gi, '<div$1').replace(/<\/p>/gi, '</div>');
  var root = new DOMParser().parseFromString(html, 'text/html');
  unWrapperTable(root);
  return root.body.innerHTML;
};

var unWrapperTable = exports.unWrapperTable = function unWrapperTable(root) {
  unWrapperTableCommon(root, function (node) {
    return node.isBlock() && (node.hasClass('bi-table') || 'table' === node.attr('data-section-key'));
  });
  NodeUtils.walkTree(root, function (node) {
    node = $(node);
    unWrapperCodeBlock(node);
  });
};

var unWrapperTableBI = exports.unWrapperTableBI = function unWrapperTableBI(root) {
  unWrapperTableCommon(root, function (node) {
    return node.isBlock() && node.hasClass('bi-table');
  });
};

var unWrapperTableCommon = exports.unWrapperTableCommon = function unWrapperTableCommon(root, check) {
  NodeUtils.walkTree(root, function (node) {
    node = $(node);
    // 编辑页面
    if (check(node)) {
      var tableNode = node.find('table');
      if (tableNode.length > 0) {
        normalizeTdContent(tableNode);
        node.replaceWith(tableNode);
        return;
      }
      // 有表格容器没有内容，直接丢掉这个节点，
      // 如果表格前面有个标题，表格和标题之间没有空隙，前面的标题双击选择的时候会带上这个空的容器
      node.remove();
    }
  });
};

var unWrapperCodeBlock = exports.unWrapperCodeBlock = function unWrapperCodeBlock(node) {
  if (node.isBlock() && 'codeblock' === node.attr('data-section-key')) {
    var value = StringUtils.decodeSectionValue(node.attr('data-section-value'));
    if (value.code) {
      var lines = value.code.split('\n');
      var p = $('<p />');
      lines.forEach(function (line) {
        var cloneLine = p.clone();
        cloneLine.html(StringUtils.escape(line).replace(/\t/g, '&nbsp;&nbsp;').replace(/\s/g, '&nbsp;'));
        node.before(cloneLine);
      });
      node.remove();
    }
  }
};

var normalizeTdContent = exports.normalizeTdContent = function normalizeTdContent(table) {
  NodeUtils.walkTree(table, function (node) {
    node = $(node);
    // 编辑页面
    if (node.isBlock()) {
      var replacer = void 0;
      switch (node.name) {
        case 'h1':
          replacer = $('<span class="zacheditor-fontsize-18"></span>');
          replacer.html(node.html());
          break;

        case 'h2':
          replacer = $('<span class="zacheditor-fontsize-16"></span>');
          replacer.html(node.html());
          break;

        case 'h3':
        case 'h4':
          replacer = $('<span class="zacheditor-fontsize-14"></span>');
          replacer.html(node.html());
          break;

        case 'h5':
        case 'h6':
          replacer = $('<span class="zacheditor-fontsize-12"></span>');
          replacer.html(node.html());
          break;

        default:
          break;
      }

      if (replacer) {
        node.replaceWith(replacer);
      }
    }

    if (node.name === 'a') {
      var url = node.attr('href') || '';
      if (!url.startsWith('#') && !node.attr('target')) {
        node.attr('target', '_blank');
      }
    }
    unWrapperCodeBlock(node);
  });
};
// firefox 下的拖拽需要这样处理
// clearData 是为了防止新开 tab
// hack: 如果不 setData, firefox 不会触发拖拽事件，但设置 data 之后，又会开新 tab, 这里设置一个 firefox 不识别的 mimetype: zacheditor
var fixDragEvent = exports.fixDragEvent = function fixDragEvent(e) {
  e.dataTransfer.clearData();
  e.dataTransfer.setData('zacheditor', null);
};

/**
 * 获得一个表格内包含内容的最大行列数
 */
var getContentArea = exports.getContentArea = function getContentArea(tableModel) {
  var table = tableModel.table,
      cols = tableModel.cols,
      rows = tableModel.rows;

  var row = rows - 1;
  var col = cols - 1;
  var rowHasContent = false;
  var colHasContent = false;

  while (row > 0 && !rowHasContent) {
    for (var c = 0; c < cols; c++) {
      if (!isTdEmpty(table, row, c)) {
        rowHasContent = true;
        break;
      }
    }

    if (!rowHasContent) {
      row--;
    }
  }

  while (col > 0 && !colHasContent) {
    for (var r = 0; r < rows; r++) {
      if (!isTdEmpty(table, r, col)) {
        colHasContent = true;
        break;
      }
    }

    if (!colHasContent) {
      col--;
    }
  }

  return {
    row: row,
    col: col
  };
};

var isTdEmpty = function isTdEmpty(table, row, col) {
  var tdModel = table[row][col];
  var td = void 0;
  if (tdModel.isEmpty) {
    td = table[tdModel.parent.row][tdModel.parent.col].element;
  } else {
    td = tdModel.element;
  }

  if (td && (td.innerText.trim() !== '' || $(td).find('img')[0])) {
    return false;
  }
  return true;
};

var fixNumberTr = function fixNumberTr(table) {
  var rows = table.rows;
  var rowCount = rows.length;
  var colCounts = [];
  var firstColCount = void 0; // 第一列的单元格个数
  var cellCounts = []; // 每行单元格个数
  var totalCellCounts = 0; // 总单元格个数
  var emptyCounts = 0; // 跨行合并缺损的单元格
  // 已经存在一行中的 td 的最大数，最终算出来的最大列数一定要大于等于这个值
  var maxCellCounts = 0; // 在不确定是否缺少tr时，先拿到已经存在的td，和一些关键信息

  for (var r = 0; r < rowCount; r++) {
    var row = rows[r];
    var cells = row.cells;
    var cellCountThisRow = 0;

    for (var c = 0; c < cells.length; c++) {
      var _cells$c = cells[c],
          rowSpan = _cells$c.rowSpan,
          colSpan = _cells$c.colSpan;

      totalCellCounts += rowSpan * colSpan;
      cellCountThisRow += colSpan;
      if (rowSpan > 1) {
        emptyCounts += (rowSpan - 1) * colSpan;
      }
    }
    cellCounts[r] = cellCountThisRow;
    if (r === 0) {
      firstColCount = cellCountThisRow;
    }
    maxCellCounts = Math.max(cellCountThisRow, maxCellCounts);
  }
  // number拷贝的一定是首行列数能被单元格总数整除
  var isNumber1 = (0, _isInteger2.default)(totalCellCounts / firstColCount); // number拷贝的一定是首行列数最大
  var isNumber2 = firstColCount === maxCellCounts;
  var isNumber = isNumber1 && isNumber2; // 判断是否是 number, 是因为 number 需要考虑先修复省略的 tr，否则后面修复出来就会有问题

  if (isNumber) {
    var lossCellCounts = 0;
    cellCounts.forEach(function (cellCount) {
      lossCellCounts += maxCellCounts - cellCount;
    });

    if (lossCellCounts !== emptyCounts) {
      var missCellCounts = emptyCounts - lossCellCounts;
      if ((0, _isInteger2.default)(missCellCounts / maxCellCounts)) {
        var lossRowIndex = []; // 记录哪一行缺 tr

        for (var _r = 0; _r < rowCount; _r++) {
          var _row = rows[_r];
          var _cells = _row.cells;
          var realRow = _r + lossRowIndex.length;

          while (colCounts[realRow] === maxCellCounts) {
            lossRowIndex.push(realRow);
            realRow++;
          }

          for (var _c2 = 0; _c2 < _cells.length; _c2++) {
            var _cells$_c = _cells[_c2],
                rowSpan = _cells$_c.rowSpan,
                colSpan = _cells$_c.colSpan;

            if (rowSpan > 1) {
              for (var rr = 1; rr < rowSpan; rr++) {
                colCounts[realRow + rr] = (colCounts[realRow + rr] || 0) + colSpan;
              }
            }
          }
        }

        lossRowIndex.forEach(function (row) {
          table.insertRow(row);
        });
      }
    }
  }
};