'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils = require('./utils');

var context = window;
var doc = document;
var byTag = 'getElementsByTagName';
var head = doc[byTag]('head')[0];
var httpsRe = /^http/;
var protocolRe = /(^\w+):\/\//;
var twoHundo = /^(20\d|1223)$/; // http://stackoverflow.com/questions/10046972/msie-returns-status-code-of-1223-for-ajax-request
var readyState = 'readyState';
var contentType = 'Content-Type';
var requestedWith = 'X-Requested-With';
var callbackPrefix = 'request_' + +new Date();
var xmlHttpRequest = 'XMLHttpRequest';
var xDomainRequest = 'XDomainRequest';
var uuid = 0;
var lastValue = undefined; // data stored by the most recent JSONP callback

var noop = function noop() {};

var getCallbackPrefix = function getCallbackPrefix(reqId) {
  return callbackPrefix + '_' + reqId;
};

var xhr = function xhr(o) {
  var ctx = o.context || context;
  // is it x-domain
  if (o.crossOrigin === true) {
    var xhrInstance = ctx[xmlHttpRequest] ? new ctx[xmlHttpRequest]() : null;
    if (xhrInstance && 'withCredentials' in xhrInstance) {
      return xhrInstance;
    }
    if (ctx[xDomainRequest]) {
      return new ctx[xDomainRequest]();
    }
    throw new Error('Browser does not support cross-origin requests');
  } else if (ctx[xmlHttpRequest]) {
    return new ctx[xmlHttpRequest]();
  } else {
    return new ctx.ActiveXObject('Microsoft.XMLHTTP');
  }
};

var globalSetupOptions = {
  traditional: false,
  contentType: 'application/x-www-form-urlencoded',
  requestedWith: xmlHttpRequest,
  accept: {
    '*': 'text/javascript, text/html, application/xml, text/xml, */*',
    xml: 'application/xml, text/xml',
    html: 'text/html',
    text: 'text/plain',
    json: 'application/json, text/javascript',
    js: 'application/javascript, text/javascript'
  },
  dataFilter: function dataFilter(data) {
    return data;
  }
};

var succeed = function succeed(r) {
  var protocol = protocolRe.exec(r.url);
  var protocolString = '';
  if (protocol) {
    protocolString = protocol[1];
  }
  if (!protocolString) {
    protocolString = (r.o.context || context).location.protocol;
  }
  return httpsRe.test(protocolString) ? twoHundo.test(r.request.status) : !!r.request.response;
};

var handleReadyState = function handleReadyState(r, success, error) {
  return function () {
    // use _aborted to mitigate against IE err c00c023f
    // (can't read props on aborted request objects)
    if (r._aborted) {
      return error('Request is aborted', r.request);
    }
    if (r._timedOut) {
      return error('Request is aborted: timeout', r.request);
    }
    if (r.request && r.request[readyState] === 4) {
      r.request.onreadystatechange = noop;
      if (succeed(r)) {
        success(r.request);
      } else {
        error(r.request.statusText, r.request);
      }
    }
  };
};

var setHeaders = function setHeaders(http, o) {
  var headers = o.headers || {};
  var h = undefined;
  headers.Accept = headers.Accept || globalSetupOptions.accept[o.type] || globalSetupOptions.accept['*'];
  // breaks cross-origin requests with legacy browsers
  if (!o.crossOrigin && !headers[requestedWith]) {
    headers[requestedWith] = globalSetupOptions.requestedWith;
  }
  if (!headers[contentType] && !(0, _utils.isAFormData)(o.data)) {
    headers[contentType] = o.contentType || globalSetupOptions.contentType;
  }
  for (h in headers) {
    if (headers.hasOwnProperty(h) && 'setRequestHeader' in http) {
      http.setRequestHeader(h, headers[h]);
    }
  }
};

var setCredentials = function setCredentials(http, o) {
  if (typeof o.withCredentials !== 'undefined' && typeof http.withCredentials !== 'undefined') {
    http.withCredentials = !!o.withCredentials;
  }
};

var generalCallback = function generalCallback(data) {
  lastValue = data;
};

function handleJsonp(o, fn, error, url_) {
  var url = url_;
  var reqId = uuid++;
  var cbkey = o.jsonpCallback || 'callback'; // the 'callback' key
  var cbval = o.jsonpCallbackName || getCallbackPrefix(reqId);
  var cbreg = new RegExp('((^|\\?|&)' + cbkey + ')=([^&]+)');
  var match = url.match(cbreg);
  var script = doc.createElement('script');
  var loaded = 0;
  var isIE10 = navigator.userAgent.indexOf('MSIE 10.0') !== -1;

  if (match) {
    if (match[3] === '?') {
      url = url.replace(cbreg, '$1=' + cbval); // wildcard callback func name
    } else {
      cbval = match[3]; // provided callback func name
    }
  } else {
    url = (0, _utils.urlAppend)(url, cbkey + '=' + cbval); // no callback details, add 'em
  }

  context[cbval] = generalCallback;

  script.type = 'text/javascript';
  script.src = url;
  script.async = true;
  if (typeof script.onreadystatechange !== 'undefined' && !isIE10) {
    // need this for IE due to out-of-order onreadystatechange(), binding script
    // execution to an event listener gives us control over when the script
    // is executed. See http://jaubourg.net/2010/07/loading-script-as-onclick-handler-of.html
    script.htmlFor = script.id = '_request_' + reqId;
  }

  script.onload = script.onreadystatechange = function () {
    if (script[readyState] && script[readyState] !== 'complete' && script[readyState] !== 'loaded' || loaded) {
      return false;
    }
    script.onload = script.onreadystatechange = null;
    if (script.onclick) {
      script.onclick();
    }
    // Call the user callback with the last value stored and clean up values and scripts.
    fn(lastValue);
    lastValue = undefined;
    head.removeChild(script);
    loaded = 1;
  };

  // Add the script to the DOM head
  head.appendChild(script);

  // Enable JSONP timeout
  return {
    abort: function abort() {
      script.onload = script.onreadystatechange = null;
      error('Request is aborted: timeout');
      lastValue = undefined;
      head.removeChild(script);
      loaded = 1;
    }
  };
}

var getRequest = function getRequest(fn, error) {
  var o = this.o;
  var method = (o.method || 'GET').toUpperCase();
  var url = typeof o === 'string' ? o : o.url;
  // convert non-string objects to query-string form unless o.processData is false
  var data = o.processData !== false && o.data && typeof o.data !== 'string' && !(0, _utils.isAFormData)(o.data) ? (0, _utils.toQueryString)(o.data, o.traditional || globalSetupOptions.traditional) : o.data || null;
  var http = undefined;
  var sendWait = false;

  // if we're working on a GET request and we have data then we should append
  // query string to end of URL and not post data
  if ((o.type === 'jsonp' || method === 'GET') && data) {
    url = (0, _utils.urlAppend)(url, data);
    data = null;
  }

  if (o.type === 'jsonp') {
    return handleJsonp(o, fn, error, url);
  }

  // get the xhr from the factory if passed
  // if the factory returns null, fall-back to ours
  http = o.xhr && o.xhr(o) || xhr(o);
  http.open(method, url, o.async === false ? false : true);
  setHeaders(http, o);
  setCredentials(http, o);
  if (context[xDomainRequest] && http instanceof context[xDomainRequest]) {
    http.onload = fn;
    http.onerror = function () {
      error('http error', http);
    };
    // NOTE: see
    // http://social.msdn.microsoft.com/Forums/en-US/iewebdevelopment/thread/30ef3add-767c-4436-b8a9-f1ca19b4812e
    http.onprogress = noop;
    sendWait = true;
  } else {
    http.onreadystatechange = handleReadyState(this, fn, error);
  }
  if (o.before) {
    o.before(http);
  }
  if (sendWait) {
    setTimeout(function () {
      http.send(data);
    }, 200);
  } else {
    http.send(data);
  }
  return http;
};

var Request = function Request() {
  this.initPromise();
};

var request = function request() {
  return new Request();
};

var setType = function setType(header) {
  // json, javascript, text/plain, text/html, xml
  if (!header) {
    return undefined;
  }
  if (header.match('json')) {
    return 'json';
  }
  if (header.match('javascript')) {
    return 'js';
  }
  if (header.match('text')) {
    return 'html';
  }
  if (header.match('xml')) {
    return 'xml';
  }
};

var init = function init(o) {
  var self = this;
  var timedOut = function timedOut() {
    self._timedOut = true;
    self.request.abort();
  };

  this.url = typeof o === 'string' ? o : o.url;

  if (this.timeout) {
    clearTimeout(this.timeout);
  }

  this.timeout = null;

  if (o.timeout) {
    this.timeout = setTimeout(timedOut, o.timeout);
  }

  var error = function error(msg, xhr_) {
    self.triggerError(msg, xhr_);
  };

  var success = function success(resp_) {
    self.triggerSuccess(resp_);
  };
  this.request = getRequest.call(this, success, error);
};

Request.prototype = {
  triggerSuccess: function triggerSuccess(resp_) {
    var o = this.o;
    var resp = resp_;
    var type = o.type;
    // use global data filter on response text
    var r = (o.dataFilter || globalSetupOptions.dataFilter)(resp.responseText, type);
    if (!type) {
      type = resp && setType(resp.getResponseHeader('Content-Type'));
    }
    // resp can be undefined in IE
    resp = type !== 'jsonp' ? this.request : resp;
    try {
      resp.responseText = r;
    } catch (e) {
      // can't assign this in IE<=8, just ignore
    }
    if (r) {
      switch (type) {
        case 'json':
          try {
            resp = context.JSON.parse(r);
          } catch (err) {
            return this.triggerError('Could not parse JSON in response', resp);
          }
          break;
        case 'html':
          resp = r;
          break;
        case 'xml':
          resp = resp.responseXML && resp.responseXML.parseError && resp.responseXML.parseError.errorCode && resp.responseXML.parseError.reason ? null : resp.responseXML;
          break;
        default:
          break;
      }
    }

    if (o.success) {
      o.success(resp);
    }
    this.triggerComplete(resp);
    this.__resolve(resp);
  },
  triggerError: function triggerError(msg, xhr_) {
    var o = this.o;
    var e = new Error(msg);
    e.xhr = xhr_;
    if (o.error) {
      o.error(e);
    }
    this.triggerComplete(e);
    this.__reject(e);
  },
  triggerComplete: function triggerComplete(resp) {
    var o = this.o;
    if (o.timeout) {
      clearTimeout(this.timeout);
    }
    this.timeout = null;
    if (o.complete) {
      o.complete(resp);
    }
  },
  initPromise: function initPromise() {
    var self = this;
    self.promise = new Promise(function (resolve, reject) {
      self.__resolve = resolve;
      self.__reject = reject;
    }).catch(function () {});
  },
  abort: function abort() {
    this._aborted = true;
    this.request.abort();
  },
  retry: function retry() {
    this.initPromise();
    init.call(this, this.o);
  },
  then: function then(success, fail) {
    return this.promise.then(success, fail);
  },
  always: function always(fn) {
    return this.promise.then(fn, fn);
  },
  fail: function fail(fn) {
    return this.promise.then(undefined, fn);
  },
  catch: function _catch(fn) {
    return this.fail(fn);
  },
  setup: function setup(o_) {
    this.o = Object.assign({}, globalSetupOptions, o_);
  },
  start: function start(o_) {
    this.setup(o_);
    init.call(this, this.o);
  }
};

request.ajaxSetup = function () {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  for (var k in options) {
    if (options.hasOwnProperty(k) && globalSetupOptions.hasOwnProperty(k)) {
      globalSetupOptions[k] = options[k];
    }
  }
};

exports.default = request;