'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.loadImage = exports.getRangesByText = exports.getTextNodes = exports.getHeight = exports.getWidth = exports.getStyle = exports.isInViewport = undefined;

var _type = require('./type');

/**
 * 获取一串字符关于另一个字符的索引集
 * @param  {String} str - 被检索字符
 * @param  {String} keyword - 用于检索的关键词
 * @return {Array} 字符的索引集
 */

var getKeywordIndexs = function getKeywordIndexs(str, keyword) {
    var rst = [];
    var l = keyword.length;
    var i = str.indexOf(keyword);

    while (i > -1) {
        rst.push(i);
        i = str.indexOf(keyword, i + l);
    }

    return rst;
};

/**
 * dom 元素是否在浏览器视口内
 * @param  {Object} dom - DOM节点
 * @return {Boolean}
 */
var isInViewport = exports.isInViewport = function isInViewport(dom) {
    var rect = dom.getBoundingClientRect();
    return rect.top >= 0 && rect.left >= 0 && rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
    /* or $(window).height() */
    rect.right <= (window.innerWidth || document.documentElement.clientWidth);
    /* or $(window).width() */
};

/**
 * 获取样式
 * @param  {Object} dom - DOM节点
 * @param  {String} name - 样式名
 * @param  {Any} defaultValue - 默认值
 * @return {String} 属性值
 */

var getStyle = exports.getStyle = function getStyle(dom, name, defaultValue) {
    try {
        if (window.getComputedStyle) {
            return window.getComputedStyle(dom, null)[name];
        }

        return dom.currentStyle[name];
    } catch (e) {
        if (!(0, _type.isNil)(defaultValue)) {
            return defaultValue;
        }
        return null;
    }
};

/**
 * 获取宽度
 * @param  {HTMLElement} el - dom节点
 * @param  {Number} defaultValue - 默认值
 * @return {Number} 宽度
 */
var getWidth = exports.getWidth = function getWidth(el, defaultValue) {
    var width = getStyle(el, 'width', defaultValue);

    if (width === 'auto') {
        width = el.offsetWidth;
    }

    return parseFloat(width);
};

/**
 * 获取高度
 * @param  {HTMLElement} el - dom节点
 * @param  {Number} defaultValue - 默认值
 * @return {Number} 高度
 */
var getHeight = exports.getHeight = function getHeight(el, defaultValue) {
    var height = getStyle(el, 'height', defaultValue);

    if (height === 'auto') {
        height = el.offsetHeight;
    }

    return parseFloat(height);
};

/**
 * 获取一个 dom 元素内所有的 textnode 类型的元素
 * @param  {HTMLElement} elem - dom节点
 * @param  {Function} filter - 过滤器
 * @return {Array} 获取的文本节点
 */
var getTextNodes = exports.getTextNodes = function getTextNodes(elem, filter) {
    var textNodes = [];
    if (elem) {
        if (filter && !filter(elem)) {
            return textNodes;
        }

        var nodes = elem.childNodes;

        for (var i = 0; i < nodes.length; i++) {
            var node = nodes[i];
            var nodeType = node.nodeType;
            if (nodeType === 3) {
                textNodes.push(node);
            } else if (nodeType === 1 || nodeType === 9 || nodeType === 11) {
                textNodes = textNodes.concat(getTextNodes(node, filter));
            }
        }
    }
    return textNodes;
};

/**
 * 从一个 testNode 集合中获取关于一个 text 的 range 集合
 * @param  {HTMLElement} textNodes - 文字元素集合
 * @param  {Function} text - 目标文本
 * @return {Array} 区间集合
 */
var getRangesByText = exports.getRangesByText = function getRangesByText(textNodes, text) {
    var ranges = [];
    textNodes.forEach(function (textNode) {
        var textValue = textNode.nodeValue;
        var l = text.length;
        var strIndexs = getKeywordIndexs(textValue, text);
        strIndexs.forEach(function (strIndex) {
            var range = document.createRange();
            range.setStart(textNode, strIndex);
            range.setEnd(textNode, strIndex + l);
            ranges.push(range);
        });
    });
    return ranges;
};

var loadImage = exports.loadImage = function loadImage(src) {
    return new Promise(function (resolve, reject) {
        var image = new Image();
        image.crossOrigin = "Anonymous";
        image.onload = function () {
            return resolve(image);
        };
        image.onerror = reject;
        image.src = src;
    });
};